/********************************************************************
	
	JavaPlugin
	
*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <algobase.h>

#include <Entry.h>
#include <Path.h>
#include <String.h>

#include <KernelExport.h>

#include "ErrorMessage.h"
#include "MProject.h"

#include "JavaPlugin.h"
#include "JavaPluginPrefs.h"

#define DBG(expr)		printf ## expr

#define TOOL_NAME		"/usr/local/bin/javac"	// !!! do not hardwire for the future


// ---------------------------------------------------------------------------

JavaPlugin::JavaPlugin()
{
	// set default prefs, not necessarily in sync with stored prefs
	strncpy(fPrefs.compiler, TOOL_NAME, MAX_STR); 
}

// ---------------------------------------------------------------------------

status_t 
JavaPlugin::GetToolName(char *outName, int32 inBufferLength, MakeStageT inStage, MakeActionT inAction)
{
	strncpy(outName, fPrefs.compiler, inBufferLength);
DBG(("ToolName: %s\n", outName));
	return B_OK;
}

// ---------------------------------------------------------------------------

const char *
JavaPlugin::LinkerName()
{
	// we don't deal with linking
	return B_EMPTY_STRING; 
}

// ---------------------------------------------------------------------------

MakeStageT 
JavaPlugin::MakeStages()
{
	// all our work is done during ompile
	return kCompileStage; 	
}

// ---------------------------------------------------------------------------

MakeActionT 
JavaPlugin::Actions()
{
	return kCompile;  
}

// ---------------------------------------------------------------------------

PlugInFlagsT 
JavaPlugin::Flags()
{
	return kNotIDEAware;
}

// ---------------------------------------------------------------------------

ulong 
JavaPlugin::MessageDataType()
{
	return kMessageType;
}

// ---------------------------------------------------------------------------


void 
JavaPlugin::ProjectChanged(ChangeT inChange, MProject* inProject)
{	
	switch (inChange)
	{
		case kProjectOpened: case kPrefsChanged:
DBG(("JavaPlugin prefs changed\n"));
			if (inProject)
			{
			BMessage msg;
				inProject->GetPrefs(kMessageType, msg);
				if (compilerPrefs::FromMessage(msg, fPrefs))
				{
DBG((" comp: %s\n path: %s\n options: %s\n strict: %d\n debug: %d\n nowarn: %d\n",
		fPrefs.compiler, fPrefs.classpath, fPrefs.options, 
		fPrefs.strict, fPrefs.debug, fPrefs.nowarn));
				}
			}
			break;
		case kProjectClosed:
			break;
		case kFilesAdded:
DBG(("Files added\n"));
			break;
		case kFilesRemoved:
		case kFilesRearranged:
		case kRunMenuItemChanged:
		case kLinkDone:
			// Don't do anything for now
			break;
		case kBuildStarted:
DBG(("Build started\n"));
#if 0
			entry_ref ref;
			if (inProject!=NULL && inProject->GetProjectRef(ref)==B_NO_ERROR)
			{	BEntry entry(&ref);
				projectPath.SetTo(&entry);
				projectPath.GetParent(&projectPath);
DBG(("Project path set to %s\n", projectPath.Path() ));
			}
#endif
			break;
	}
}

// ---------------------------------------------------------------------------

bool 
JavaPlugin::ValidateSettings(BMessage &inOutMessage)
{
	return false;
}

// ---------------------------------------------------------------------------

status_t 
JavaPlugin::BuildPrecompileArgv(BList &inArgv, MFileRec &inFileRec)
{
/*
	if (inFileRec.makeStage != kPrecompileStage) {
		return B_ERROR;
	}

	return BuildArgv(inArgv, inFileRec.path);
*/
	return B_ERROR;
}

// ---------------------------------------------------------------------------

status_t 
JavaPlugin::BuildCompileArgv(BList &inArgv, MakeActionT inAction, MFileRec &inFileRec)
{
	if (inFileRec.makeStage != kCompileStage) {
		return B_ERROR;
	}

	char * filePath = inFileRec.path;
	
	//BString outputName;
	//this->MakeOutputFileName(filePath, outputName);

	// -- pizza argument list
	if (strlen(fPrefs.classpath) > 0)
	{	
		inArgv.AddItem(strdup("-classpath"));
		inArgv.AddItem(strdup(fPrefs.classpath));
	}
	if (fPrefs.debug)
		inArgv.AddItem(strdup("-debug"));
	if (fPrefs.strict)
		inArgv.AddItem(strdup("-java"));
	if (fPrefs.nowarn)
		inArgv.AddItem(strdup("-nowarn"));
	if (strlen(fPrefs.options) > 0)
		inArgv.AddItem(strdup(fPrefs.options));
	
	// -- file to compile	
	inArgv.AddItem(strdup(filePath));
	return B_OK;
}

// ---------------------------------------------------------------------------

status_t 
JavaPlugin::BuildPostLinkArgv(BList &inArgv, MFileRec &inFileRec)
{
	// we don't do anything in this stage
	return B_ERROR;
}

// ---------------------------------------------------------------------------

bool 
JavaPlugin::FileIsDirty(MFileRec &inFileRec, MakeStageT inStage, MakeActionT inAction, time_t inModDate)
{
	// See if inFile is newer than its output file
	
	BString outputFileName;
	MakeOutputFileName(inFileRec.path, outputFileName);
	
	// assume the best...
	bool isDirty = false;
		
	BEntry outputFile(outputFileName.String());
	if (outputFile.Exists() == false || outputFile.IsFile() == false) {
		// if we can't find the output file or it isn't a file
		// we have to assume the source is dirty
DBG(("File %s doesn't exist\n", outputFileName.String() ));
		isDirty = true;
	}
	else {
		time_t outputModDate;
		outputFile.GetModificationTime(&outputModDate);
		if (outputModDate < inModDate) {
			// if the output file is older than our input file
			// it is dirty
DBG(("File %s is dirty\n", outputFileName.String() ));
			isDirty = true;
		}
	}
	return isDirty;	
}

// ---------------------------------------------------------------------------

void 
JavaPlugin::CodeDataSize(const char *inFilePath, int32 &outCodeSize, int32 &outDataSize)
{
	outCodeSize = -1;  // !!! FIXME
	outDataSize = -1;  // !!! FIXME
}

// ---------------------------------------------------------------------------

status_t 
JavaPlugin::GenerateDependencies(const char *inFilePath, BList &outList)
{
	// we are not dependent on anything, our output file is
	// but we aren't called for that one, the compiler is (or whatever tool
	// is used to include it in the build)
	return B_ERROR;
}

// ---------------------------------------------------------------------------

void 
JavaPlugin::GetTargetFilePaths(MFileRec &inFileRec, BList &inOutTargetFileList)
{
	// this is used by the linker to generate the list of object files to link
	// not applicable for tools

	//BString outputFileName;
	//MakeOutputFileName(inFileRec.name, outputFileName);
	//inOutTargetFileList.AddItem(strdup(outputFileName.String()));
}


// ---------------------------------------------------------------------------

void 
JavaPlugin::MakeOutputFileName(const char *filePath, BString &outputName)
{
	// take the input filePath and strip off the last .java and put on a .class
	outputName = filePath;
/**
	BPath file(filePath);
	outputName = projectPath.Path();
	outputName.Append("/");
	outputName.Append(file.Leaf());
**/
	outputName.ReplaceLast(".java", ".class");
DBG(("Output file path: %s\n", outputName.String() ));
}


// ---------------------------------------------------------------------------

const char kNewLine = '\n';

status_t 
JavaPlugin::ParseMessageText(const char *inText, BList &outList)
{	
	// put the text into a BString and make sure it ends with a newline
	// for less edge case testing below...
	BString text(inText);
	if (text[text.Length()-1] != kNewLine) {
		text += kNewLine;
	}

	// !!! BURP, BAD HACK TO GROUP ERROR LINES TOGETHER
	bool isJavac = (strstr(fPrefs.compiler, "javac") != NULL) 
					|| (strstr(fPrefs.compiler, "pizza") != NULL); 
	
	if (isJavac)
	{
DBG(("javac/pizza error reporting\n"));
		// iterate through the text 
		int32 lineEnd = 0, tempEnd = 0;
		int32 lineStart = 0, tempStart = 0;
		int   lineCnt=0;
		while ((lineEnd = text.FindFirst(kNewLine, lineStart)) != -1) {
			// skip empty lines
			if (lineEnd > lineStart + 1) {
			
				// skip 2 more lines
				tempStart = lineEnd+1;
				for(lineCnt=0; lineCnt<2; lineCnt++)
				{
					if ((tempEnd = text.FindFirst(kNewLine, tempStart)) == -1) break;
					tempStart = tempEnd + 1;
				}
								
//DBG(("[Err %ld -> %ld .. %ld]\n", lineStart, lineEnd, tempEnd));
				BString errLines;
				text.CopyInto(errLines, lineStart, lineEnd-lineStart);
				
				if (lineCnt<2)
				{	// !!! hack to filter warning and error counts
					bool isNWarn = (lineEnd-lineStart >= 8) 
								    && (errLines.FindFirst("warning", 0) != -1);
					bool isNErr  = (lineEnd-lineStart >= 6)
									&& (errLines.FindFirst("error", 0) != -1);
DBG(("%d lines, warn: %d, err: %d]\n", lineCnt, isNWarn, isNErr));
					if (! (isNWarn || isNErr))
						outList.AddItem( CreateTextOnlyMessage(errLines) );
				}
				else
					outList.AddItem(ParseFileLineError(errLines, "", ":", "", ":"));
			}
			lineStart = max(lineEnd, tempEnd) + 1;
		}	
	}
	else
	{
DBG(("generic error reporting\n"));
		// iterate through the text one line at a time 
		int32 lineEnd = 0;
		int32 lineStart = 0;
		while ((lineEnd = text.FindFirst(kNewLine, lineStart)) != -1) {
			// skip empty lines
			if (lineEnd > lineStart + 1) {
				BString oneLine;
				
				text.CopyInto(oneLine, lineStart, lineEnd-lineStart);
// !!! FIXME    ParseFileLineError chokes on guavac errors
//				outList.AddItem(ParseFileLineError(oneLine, "", ":", "", ":"));
				outList.AddItem( CreateTextOnlyMessage(oneLine) );
			}
			lineStart = lineEnd + 1;
		}
	}
		
	return B_OK;
}

// ---------------------------------------------------------------------------

ErrorMessage*
JavaPlugin::CreateEmptyError()
{	
	ErrorMessage* error = new ErrorMessage;
	// -1 is a sentinal to trigger the creation of dynamic offsets
	// in MMessageItem.  (With an offset and length, the error message
	// will try to track edits made to the file.)
	error->offset = -1;
	error->length = 0;
	error->synclen = 0;
	error->syncoffset = 0;
	error->errorlength = 0;
	error->erroroffset = 0;
	
	error->filename[0] = 0;
	error->errorMessage[0] = 0;
	error->errorMessage[MAX_ERROR_TEXT_LENGTH-1] = 0;
	
	// the default (until we correctly parse the filename/linenumber) is text only
	error->linenumber = 0;
	error->textonly = true;
	error->isWarning = false;
	
	return error;
}

// ---------------------------------------------------------------------------

ErrorMessage*
JavaPlugin::CreateTextOnlyMessage(const BString& text)
{
	ErrorMessage* error = this->CreateEmptyError();
	int32 amount = min(text.Length(), MAX_ERROR_TEXT_LENGTH-1);
	text.CopyInto(error->errorMessage, 0, amount);
	error->errorMessage[amount] = 0;

	return error;
}

// ---------------------------------------------------------------------------

const char * WARN_STR = " warning:";

ErrorMessage*
JavaPlugin::ParseFileLineError(const BString& text,
								const char* filePrefix, 
								const char* fileSuffix, 
								const char* linePrefix, 
								const char* lineSuffix)
{
	// Look for the following in the text:
	//	filePrefix<fileName>fileSuffix
	// 	linePrefix<lineNumber>lineSuffix
	// If we don't succeed in finding everything, create
	// a text only error message with the full text
	// as the error text
	
	// rather than deeply nesting, I return at text only
	// message at the first problem encountered...
	
	int32 fileStart = 0;
#if 0
	fileStart = text.FindFirst(filePrefix);
	if (fileStart == -1) {
		return this->CreateTextOnlyMessage(text);
	}
	fileStart += strlen(filePrefix);
#endif
//DBG(("[file start %ld]", fileStart));

	int32 fileEnd = text.FindFirst(fileSuffix, fileStart);
	if (fileEnd == -1) {
		return this->CreateTextOnlyMessage(text);
	}
//DBG(("[file end %ld]", fileEnd));
	
	int32 lineStart;
#if 0
	lineStart = text.FindFirst(linePrefix, fileEnd);
	if (lineStart == -1) {
		return this->CreateTextOnlyMessage(text);
	}
#else
	lineStart = fileEnd+1;
#endif
	lineStart += strlen(linePrefix);
//DBG(("[line start %ld]", lineStart));

	int32 lineEnd = text.FindFirst(lineSuffix, lineStart);
	if (lineEnd == -1 /*|| text[lineStart]==' '*/) {
		return this->CreateTextOnlyMessage(text);
	}
//DBG(("[line end %ld]", lineEnd));
	
	// we found the file and line
	// set up the file/line in our error message and
	// copy only the remaining text as the message
	ErrorMessage* error = this->CreateEmptyError();

	int32 length;

	// handle the file name
	length = min(fileEnd-fileStart, MAX_ERROR_PATH_LENGTH-1);
	text.CopyInto(error->filename, fileStart, length);
	error->filename[length] = 0;
	error->textonly = false;
//DBG((" File: %s", error->filename));

#if 1
	// ...the line number
	char lineNumberBuffer[32];
	length = min(lineEnd-lineStart, (int32) 32);
	text.CopyInto(lineNumberBuffer, lineStart, length);
	lineNumberBuffer[length] = 0;
	error->linenumber = atoi(lineNumberBuffer);
//DBG((" Line: %ld\n", error->linenumber));

	// ...and the error message text
	int32 messageStart = lineEnd + strlen(lineSuffix);
	
	// ...is it a warning?
	int32 warningStart = text.IFindFirst(WARN_STR, messageStart);
	if (warningStart == messageStart)
	{ 
	  error->isWarning = true;
	  messageStart += strlen(WARN_STR);
	}
	// ...copy message text
	length = min(text.Length()-messageStart, MAX_ERROR_TEXT_LENGTH-1);
	text.CopyInto(error->errorMessage, messageStart, length);
#else
	length=min(text.Length(), MAX_ERROR_TEXT_LENGTH-1);
	text.CopyInto(error->errorMessage, 0, length);
#endif
	error->errorMessage[length] = 0;
	return error;
}

