/*********************************************************************	

	JavaPlugin
     Implements MPlugInBuilder as defined in Plugin API

*********************************************************************/

#ifndef JAVA_PLUGIN_H
#define JAVA_PLUGIN_H

#include "MPlugInBuilder.h"
#include "ErrorMessage.h"

#include "JavaPluginPrefs.h"


class JavaPlugin : public MPlugInBuilder
{
public:
								JavaPlugin();
									
	virtual status_t			GetToolName(
									char* 		outName,
									int32		inBufferLength,
									MakeStageT	inStage,
									MakeActionT	inAction);

	virtual const char *		LinkerName();
	virtual MakeStageT			MakeStages();
	virtual MakeActionT			Actions();
	virtual PlugInFlagsT		Flags();
	virtual ulong				MessageDataType();

	// return true if something changed in the settings
	virtual bool				ValidateSettings(
									BMessage&	inOutMessage);

	// return B_NO_ERROR if everything went OK
	virtual status_t			BuildPrecompileArgv(
									BList& 		inArgv,
									MFileRec& 	inFileRec);

	virtual status_t			BuildCompileArgv(
									BList& 		inArgv,
									MakeActionT inAction,
									MFileRec& 	inFileRec);

	virtual status_t			BuildPostLinkArgv(
									BList& 		inArgv,
									MFileRec& 	inFileRec);

	virtual bool				FileIsDirty(
									MFileRec& 	inFileRec,
									MakeStageT	inStage,
									MakeActionT	inAction,
									time_t		inModDate);

	virtual status_t			ParseMessageText(
									const char*	inText,
									BList&		outList);
	virtual void				CodeDataSize(
									const char* inFilePath,
									int32&		outCodeSize,
									int32&		outDataSize);
	virtual status_t			GenerateDependencies(
									const char* inFilePath,
									BList&		outList);

	virtual void				GetTargetFilePaths(
									MFileRec& 	inFileRec,
									BList&		inOutTargetFileList);

	virtual void				ProjectChanged(
									ChangeT		inChange,
									MProject*	inProject);

private:
	MProject		*fProject;
	compilerPrefs	 fPrefs;
	
	void						MakeOutputFileName(
									const char *filePath, 
									BString &outputName);
									
	ErrorMessage*				ParseFileLineError(const BString& text,
									const char* filePrefix, 
									const char* fileSuffix, 
									const char* linePrefix, 
									const char* lineSuffix);
									
	ErrorMessage*				CreateEmptyError();
	ErrorMessage*				CreateTextOnlyMessage(const BString& text);

};

#endif /* JAVA_PLUGIN_H */
