/*********************************************************************	

	JavaPluginPrefs

*********************************************************************/

#include <string.h>
#include "JavaPluginPrefs.h"

#include <stdio.h>
#include <string.h>
#define DBG(expr)	printf##expr

const int lwidth = 130;
const char * strComp     = "Compiler: ";
const char * strClasspath= "Class pathname: ";	
const char * strOpt      = "Additional compiler options: ";
const char * strNowarn   = "Suppress warning messages";
const char * strDebug    = "Include debug information";
const char * strStrict   = "Strict Java input";

#define DEFAULT_TOOL	"/usr/local/bin/javac"

/********************************************************************/

JavaPluginPrefs::JavaPluginPrefs(BRect inFrame, const char*	inName, ulong inResizeMask, ulong inFlags)
  : MPlugInPrefsView(inFrame, inName, inResizeMask, inFlags)
{
}

/********************************************************************/

enum {	kChanged = 'JCng' };

void
JavaPluginPrefs::AttachedToWindow()
{
	SetViewColor(kPrefsGray);  

	BBox * group = new BBox(Bounds());
	if (group == NULL) return;
		
	group->SetLabel(Title());
	AddChild(group);	
	
	BRect r(20, 30, Bounds().right-20, 40);
	fCompiler = new BTextControl( r, NULL, strComp, DEFAULT_TOOL, new BMessage(kChanged));
	fCompiler->SetTarget(this);
	fCompiler->SetModificationMessage(new BMessage(kChanged));
	if (fCompiler) fCompiler->SetDivider(lwidth);
	group->AddChild(fCompiler);
	
	r.OffsetBy(0, 40);
	fClasspath = new BTextControl(r, NULL, strClasspath, "", new BMessage(kChanged));
	fClasspath->SetTarget(this);
	fClasspath->SetModificationMessage(new BMessage(kChanged));
	if (fClasspath) fClasspath->SetDivider(lwidth);
	group->AddChild(fClasspath);

	r.OffsetBy(0, 40);
	fOptions  = new BTextControl(r, NULL, strOpt, "", new BMessage(kChanged));
	fOptions->SetTarget(this);
	fOptions->SetModificationMessage(new BMessage(kChanged));
	if (fOptions) fOptions->SetDivider(lwidth);
	group->AddChild(fOptions);

	r.OffsetBy(10, 40);
	fStrict = new BCheckBox(r, NULL, strStrict, new BMessage(kChanged));
	fStrict->SetTarget(this);
	group->AddChild(fStrict);

	r.OffsetBy(0, 20);
	fNowarn = new BCheckBox(r, NULL, strNowarn, new BMessage(kChanged));
	fNowarn->SetTarget(this);
	group->AddChild(fNowarn);
	
	r.OffsetBy(0, 20);
	fDebug = new BCheckBox(r, NULL, strDebug, new BMessage(kChanged));
	fDebug->SetTarget(this);
	group->AddChild(fDebug);
}

/********************************************************************/
void
JavaPluginPrefs::MessageReceived(BMessage * msg)
{
	char * v = (char *)&msg->what;
#if B_HOST_IS_LENDIAN
	DBG(("MessageReceived '%c%c%c%c'\n", v[3],v[2],v[1],v[0]));
#endif	

	switch(msg->what)
	{
	case kChanged:
		ValueChanged();
		break;
	default:
		MPlugInPrefsView::MessageReceived(msg);
	}
}

/********************************************************************/
const char *		
JavaPluginPrefs::Title()
{
	return "Java Compiler";
}

/********************************************************************/
TargetT				
JavaPluginPrefs::Targets()
{
	return kMWDefaults + kCurrentProject;
}

/********************************************************************/
void
JavaPluginPrefs::GetPointers( void*&	outOld, void*&	outNew, long&	outLength)
{
	outOld = &fOld;
	outNew = &fNew;
	outLength = sizeof(fNew);
}

/********************************************************************/
void
JavaPluginPrefs::GetData(BMessage&	ioMsg)
{
	DBG(("GetData\n"));

	fNew.version = kPrefsVersion;
	fNew.SwapHostToBig();		// prefs are stored big endian in the project
	ioMsg.AddData(kPrefsName, kMessageType, &fNew, sizeof(fNew));
	fNew.SwapBigToHost();
}

/********************************************************************/
bool
compilerPrefs::FromMessage(BMessage & inMsg, compilerPrefs & outPrefs)
{
	compilerPrefs * prefs;
	ssize_t size = sizeof(outPrefs);
	if (inMsg.FindData(kPrefsName, kMessageType, (const void **)&prefs, &size) == B_NO_ERROR)
	{
		memset(&outPrefs, 0, sizeof(compilerPrefs));
		outPrefs = *prefs;
		outPrefs.SwapBigToHost();
		return true;
	}
	return false;
}

void
JavaPluginPrefs::SetData(BMessage&	ioMsg)
{

DBG(("SetData\n"));
	if (compilerPrefs::FromMessage(ioMsg, fNew))
	{
		fOld = fNew;
		UpdateValues();
	}
/**
	compilerPrefs * prefs;
	ssize_t size = sizeof(fNew);
	if (ioMsg.FindData(kPrefsName, kMessageType, (const void **)&prefs, &size) == B_NO_ERROR)
	{
		fNew = *prefs;
		fNew.SwapBigToHost();
		fOld = fNew;

		UpdateValues();	
	} 
**/
	
}

/********************************************************************/
void
JavaPluginPrefs::UpdateValues()
{
	DBG(("UpdateValues\n"));
	fCompiler->SetText(fNew.compiler);
	fClasspath->SetText(fNew.classpath);
	fOptions->SetText(fNew.options);
	fDebug->SetValue(fNew.debug);
	fStrict->SetValue(fNew.strict);
	fNowarn->SetValue(fNew.nowarn);
}

/********************************************************************/
void
JavaPluginPrefs::ValueChanged()
{
	DBG(("ValueChanged\n"));
	strncpy(fNew.compiler, fCompiler->Text(), MAX_STR);
	strncpy(fNew.classpath, fClasspath->Text(), MAX_STR);
	strncpy(fNew.options, fOptions->Text(), MAX_STR);
	fNew.debug = fDebug->Value();
	fNew.strict= fStrict->Value();
	fNew.nowarn= fNowarn->Value();
		
	Window()->PostMessage(msgPluginViewModified);
}

/********************************************************************/
void 
JavaPluginPrefs::DoSave()
{
	DBG(("DoSave\n"));
}

/********************************************************************/
void
JavaPluginPrefs::DoRevert()
{
	DBG(("DoRevert\n"));
}

/********************************************************************/
void
JavaPluginPrefs::DoFactorySettings()
{
	DBG(("DoFactorySettings\n"));
	fCompiler->SetText(DEFAULT_TOOL);
	fClasspath->SetText("");
	fOptions->SetText("");
	fStrict->SetValue(false);
	fDebug->SetValue(false);
	fNowarn->SetValue(false);
	ValueChanged();
}

/********************************************************************/
bool
JavaPluginPrefs::FilterKeyDown( ulong aKey)
{
	DBG(("FilterKeyDown\n"));
	return true;	// we pass keys to the target view
}
	
/********************************************************************/
bool
JavaPluginPrefs::ProjectRequiresUpdate(UpdateType inType)
{
	DBG(("ProjectRequiresUpdate\n"));
	return inType == kCompileUpdate;
}
