/*********************************************************************	

	JavaPluginPrefs
     Implements MPlugInPrefsView as defined in Plugin API

*********************************************************************/

#ifndef JAVA_PLUGIN_PREFS_H
#define JAVA_PLUGIN_PREFS_H

#include <InterfaceKit.h>
#include "MPlugInPrefsView.h"

#define MAX_STR		255

#define	kPrefsName	"JavcPrefs"
#if B_HOST_IS_BENDIAN
const ulong kMessageType = 'Javc';
#else
const ulong kMessageType = 'cvaJ';
#endif


const int32 kPrefsVersion = 1;

/*---------------------------------------------------------
   low-level structure
----------------------------------------------------------*/
struct compilerPrefs
{		
	static bool FromMessage(BMessage & inMsg, compilerPrefs & outPrefs);

	// BeIDE saves prefences as big endian data
	void SwapBigToHost()	
		{	version = B_BENDIAN_TO_HOST_INT32(version); 
			
		}
	
	int32	version;
	
	char	compiler [MAX_STR+1];
	char	classpath[MAX_STR+1];
	char	options  [MAX_STR+1];
	// -- pizza specific
	bool	strict;	
	bool	debug;
	bool	nowarn;
	// -- guavac specific
	// ...
	// -- jikes specific
	// ...

	void SwapHostToBig() 
		{
			version = B_HOST_TO_BENDIAN_INT32(version);
		}

};

/*---------------------------------------------------------
   JavaPluginPrefs
----------------------------------------------------------*/
class JavaPluginPrefs : public MPlugInPrefsView
{

public:

	JavaPluginPrefs(	BRect			inFrame,
						const char*		inName,
						ulong			inResizeMask,
						ulong			inFlags);

	// -- user interaction
	virtual void				AttachedToWindow();

	virtual void				MessageReceived(
									BMessage*	inMessage);
									
	// -- abstract methods implemented	
	virtual const char *		Title();
	virtual TargetT				Targets();

	virtual void				GetPointers(
									void*&	outOld,
									void*&	outNew,
									long&	outLength);
	virtual void				GetData(
									BMessage&	inOutMessage);
	virtual void				SetData(
									BMessage&	inOutMessage);
	
	virtual void				DoSave();
	virtual void				DoRevert();
	virtual void				DoFactorySettings();
	
	virtual void				UpdateValues();
	virtual void				ValueChanged();

	virtual	bool				FilterKeyDown(
									ulong aKey);	
	virtual	bool				ProjectRequiresUpdate(
									UpdateType inType);

private:
	BTextControl * fCompiler, * fClasspath, * fOptions;
	BCheckBox    * fNowarn, * fDebug, * fStrict;
	
	compilerPrefs	fNew, fOld;
};

#endif /* JAVA_PLUGIN_PREFS_H */